table.gipfl-name-value-table {
  width: 100%;
  > tbody > tr > th {
    color: @text-color-light;
    // Reset default font-weight
    font-weight: bold;
    padding-left: 0;
    text-align: right;
    vertical-align: top;
    width: 12em;
  }
  > tbody > tr > td {
    vertical-align: top;
  }
}

table.gipfl-name-value-table a {
  color: @icinga-blue;
}

table.gipfl-name-value-table pre {
  background-color: transparent;
  margin: 0;
  padding: 0;
  font-size: unset;
  display: inline;
}
