<?php
/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */
namespace gipfl\ZfDb\Adapter\Exception;

use Exception;

/**
 * Zend_Db_Adapter_Exception
 */

/**
 * Zend_Db_Adapter_Db2_Exception
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class AdapterExceptionDb2 extends AdapterException
{
    protected $code = '00000';
    protected $message = 'unknown exception';

    public function __construct($message = 'unknown exception', $code = '00000', Exception $e = null)
    {
        parent::__construct($message, $code, $e);
    }
}
