form.gipfl-quicksearch {
    display: block;
    input.search {
        width: 8em;
        min-width: unset;
        border: none;
        background-size: 1em auto;
        background-position: 0.5em 0;
        padding-left: 2em;
        margin-left: 1.5em;
        font-size: 0.75em;
        font-weight: normal;
        &:focus {
            width: 16em;
            .transition(width 0.5s ease);
            border: none;
        }
    }
}

#layout.twocols form.gipfl-quicksearch input.search {
    float: right;
}
